{% block geojs_viewer %}

{% if original.pk %}

<div class="wrapper" style="width: 100%; height: 500px;">
  {% include 'django_large_image/_include/base.html' %}
  <script>
    var imageId = {{ original.pk }};
    if (host === undefined) {
      host = window.location.protocol + "//" + window.location.host;
    }
    tileUrl = `${host}/${baseEndpoint}/${imageId}/tiles/{z}/{x}/{y}.png`;
    thumbnailUrl = `${host}/${baseEndpoint}/${imageId}/thumbnail.png`;
  </script>
  {% include 'django_large_image/_include/geojs.html' %}
</div>

{% endif %}

{% endblock %}
