{% extends "django_large_image/base.html" %}
{% block content %}
<script>
  updateTileUrlOption('projection', 'EPSG:3857');
</script>

<div class="wrapper">
  {% include 'django_large_image/_include/cesium.html' %}
</div>

{% endblock content %}
